import { Component, OnInit, Input, forwardRef } from '@angular/core';
import { ControlValueAccessor, NG_VALUE_ACCESSOR } from '@angular/forms';

export interface ValueTransformer {
	inputFn: (value: string) => { modifier: string, value: string }
	outputFn: (modifier: string, value: string) => string
}

@Component({
	selector: 'app-dropdown-modified-input',
	templateUrl: 'dropdown-modified-input.component.html',
	styles: [],
	providers: [
		{ 
		  provide: NG_VALUE_ACCESSOR,
		  useExisting: forwardRef(() => DropdownModifiedInputComponent),
		  multi: true
		}
	]
})
export class DropdownModifiedInputComponent implements OnInit, ControlValueAccessor {


	@Input()
	meta: StringMap= {};

	modifiers: Array<string>;
	transform: ValueTransformer = {
		inputFn: value => ({ modifier: '', value: value }),
		outputFn: (modifier, value) => value
	};
	private selectedModifier: string;
	private displayedValue: string;
	private _controlValue: string;
	
	private extraClass: string = '';

	propagateChange = (_: any) => {};

	ngOnInit() {
		// Move meta variables up a level, for direct access in templates
		['modifiers', 'transform', 'placeholder'].map(p => this[p] = this.meta[p] || this[p]);
		this.controlValue = this.meta.value;
	}
	
	get controlValue(): string {
		return this._controlValue;;
	}

	set controlValue(inputValue) {
		const { modifier, value } = this.transform.inputFn(inputValue);
		this.selectedModifier = modifier;
		this.displayedValue = value;
		this._controlValue = inputValue;
	}
	
	writeValue(value: string): void {
		this.controlValue = value;
	}

	registerOnChange(fn: any): void {
		this.propagateChange = fn;
	}

	registerOnTouched(fn: any): void {
	}

	modifierChange(modifier) {
		this.selectedModifier = modifier;
		this._controlValue = this.transform.outputFn(this.selectedModifier, this.displayedValue);
		this.propagateChange(this.controlValue);
	}

	inputChange() {
		this._controlValue = this.transform.outputFn(this.selectedModifier, this.displayedValue);
		this.propagateChange(this.controlValue);
	}

}
